/*
*	kitchen timer by Norbert Bokor "glezmen" <glezmen@gmail.com>
*
*	Display is a 2 digit common cathode 7 segment LCD (Kingbright DC03-11GWA)
*	Can be controlled via 3 microswitches (connected via pullup resistors):
*		UP  	to increase time set
*		DOWN	to decrease time set
*		START/STOP	to start or stop timer
*	Blinking of a status led shows if timer is running
*	Last time set will be saved to eeprom
*/

#include	<EEPROM.h>
#include	<avr/sleep.h>

#define		DEFAULTMIN	10             //	default value for target minute
#define		ALARMSECS	60             //	maximum seconds for alarm - after that many secs it's stopped
#define		PRESSED		LOW            //	state on pin when button is pressed
#define		BUTTONDELAY	500            //	msecs between two time increase/decrease with buttons
#define		DIGITDELAY	6              //	msecs to wait between display of two digits
#define		SLEEPTIME	120            //	seconds after Arduino goes to sleep
#define		PRELLCOUNT	10             //	how many times to check for prelling before accepting
#define		EEPROMPOS	178            //	position in EEPROM where last time set is stored - intentionally not using common values like 0 or 1 :)

const byte pinDigit[]     = {10,11};   //	pins for 7 segment LCD digit selection, segment control will be on pins 0-6
const byte pinFirstSegment = 3;        //	first pin for segment control (the other 6 pins will directly follow this one)
const byte pinButtonUp    =  0;        //	microswitch for UP direction (increase numbers)
const byte pinButtonDown  =  1;        //	microswitch for DOWN direction (decrease numbers)
const byte pinButtonStart =  2;        //	microswitch for Start/Stop
const byte pinSpeaker     = 12;        //	pin for alarm buzzer
const byte pinLed         = 13;        //	pin for status LED (hey, we have a LED on pin13 by default :D)

unsigned long finish      =  0;        //	timestamp in msecs for alarm, will be set when timer is started, 0 when timer is stopped
unsigned long lastpress   =  0;        //	timestamp in msecs of last button press (used for sleep timing)
byte       targetmin;                  //	minutes set

/*
	pin connections:               Arduino pins on display
	display	arduino             
       1       5                              4
       2       not connected                 3 6
       3       4                              5
       4       3                             8 7
       5       11 (digit 2)                   9
       6       9
       7       8
       8       7
       9       6
       10      10 (digit 1)
*/
//	assign segments to numbers
const byte segments[][7] =
{
//   3 4 5 6 7 8 9 <-- Arduino pins
	{1,1,0,1,1,1,1},	//	number 0
	{0,0,0,1,1,0,0},	//	number 1
	{0,1,1,1,0,1,1},	//	number 2
	{0,1,1,1,1,0,1},	//	number 3
	{1,0,1,1,1,0,0},	//	number 4
	{1,1,1,0,1,0,1},	//	number 5
	{1,1,1,0,1,1,1},	//	number 6
	{0,1,0,1,1,0,0},	//	number 7
	{1,1,1,1,1,1,1},	//	number 8
	{1,1,1,1,1,0,1} 	//	number 9
};

void setup()
{
	//	setting segment pins 0-6 to output
	for (byte i=0; i<=6; ++i)
		pinMode(pinFirstSegment + i, OUTPUT);
	pinMode(pinSpeaker,  OUTPUT);
	pinMode(pinLed,      OUTPUT);
	pinMode(pinDigit[0], OUTPUT);
	pinMode(pinDigit[1], OUTPUT);
	/*
	we could set the button pins to INPUT, but that's the
	default setting, so let's save a few bytes of flash :)
	pinMode(pinButtonUp,    INPUT);
	pinMode(pinButtonDown,  INPUT);
	pinMode(pinButtonStart, INPUT);
	*/

	//	reading last time set from EEPROM
	targetmin = EEPROM.read(EEPROMPOS);
	if (targetmin > 99)
		targetmin = DEFAULTMIN;
}

/*
	Returns TRUE if button on parameter pin is pressed

	Uses the PRESSED define, which should be LOW for
	pullup resistor, HIGH for pulldown.

*/
bool isPressed(byte pin)
{
	//	to avoid prelling, we check the state several times,
	//	and accept it as pressed only if all checks are OK
	for (byte i=0; i<PRELLCOUNT; ++i)
	{
		if (digitalRead(pin) != PRESSED)
			return false;
		delay(1);
	}
	lastpress = millis();
	return true;
}

/*
	Countdown finished, play the 'demo' for alarm until Start/Stop
	button is pressed or timeout
*/
void alarm()
{
	unsigned long started = millis();
	while (!isPressed(pinButtonStart) && (millis() - started < 1000L * ALARMSECS))
		demo();
	//	if the alarm was not stopped by pressing the button, light up the LED, otherwise switch it off
	digitalWrite(pinLed, !isPressed(pinButtonStart));
	while (isPressed(pinButtonStart))
		;	//	wait until button is released
}

/*
	Showing some demo using all the output devices
*/
void demo()
{
	for (byte i=0; i<=6; ++i)
		digitalWrite(pinFirstSegment + i, HIGH);
	digitalWrite(pinLed, (millis()/1000)%2);
	byte h = millis()%1000<500;
	digitalWrite(pinDigit[h], HIGH);
	digitalWrite(pinDigit[1-h], LOW);
	if (millis()%500<250)
		digitalWrite(pinSpeaker, HIGH);
	delayMicroseconds(400);
	digitalWrite(pinSpeaker, LOW);
	delayMicroseconds(100);
}

/*
	display ONE number on the selected digit
	digit parameter is 0 for first digit, 1 for second digit
*/
void displayDigit(byte digit, byte value)
{
	digitalWrite(pinDigit[digit],   LOW);
	digitalWrite(pinDigit[1-digit], HIGH);
	for (byte i=0; i<7; ++i)
		digitalWrite(pinFirstSegment + i, segments[value][i]);
}

/*
	Show num on 7segment display
*/
void display(byte num)
{
	if (num>9)
	{
		displayDigit(0, num/10);
		delay(DIGITDELAY);
	}
	displayDigit(1, num%10);
	delay(DIGITDELAY);
}

/*
	Called on wakeup interrupt
*/
void wakeUp()
{
	//	nothing to do
}

/*
	Sends the Arduino to sleep state to save energy
*/
void goSleep()
{
	//	switch off all outputs
	for (byte i=0; i<7; ++i)
		digitalWrite(pinFirstSegment + i, LOW);
	digitalWrite(pinDigit[0], LOW);
	digitalWrite(pinDigit[1], LOW);
	digitalWrite(pinLed, LOW);

	set_sleep_mode(SLEEP_MODE_PWR_DOWN);
	sleep_enable();
	//	wakeup connected to interrupt 0, which is on pin2 -> Start/Stop button
	attachInterrupt(0, wakeUp, LOW);
	sleep_mode();
	/*    ARDUINO SLEEPS HERE    */
	sleep_disable();
	detachInterrupt(0);
	while (isPressed(pinButtonStart))
		;	//	wait until button is released
}

void loop()
{
	unsigned long now = millis();           //	current timestamp (to avoid many calls to minis())
	static unsigned long lastChange = 0;    //	storing timestamp of last up/down button press/check

	if (now >= finish)
	{	/*    stopped or alarm    */
		if (finish)
		{	//	countdown is over
			alarm();
			lastpress = now;	//	we want to stay awake for a while after alarm, even if button was not pressed to stop it
			finish = 0;
		}

		//	buttons not pressed since a long time, and countdown is not running
		if (now > lastpress + 1000L*SLEEPTIME)
			goSleep();

		if (isPressed(pinButtonUp) || isPressed(pinButtonDown))
		{	/*    Up/Down button pressed    */
			if (now > lastChange + BUTTONDELAY)
			{
				if (isPressed(pinButtonUp) && targetmin < 99)
					++targetmin;
				else if (isPressed(pinButtonDown) && targetmin > 1)
					--targetmin;
				lastChange = now;
			}
		}
		else
			lastChange = 0;	//	buttons are released, allow time increase on next press
		display(targetmin);

		//	start timer
		if (isPressed(pinButtonStart))
		{
			while (isPressed(pinButtonStart))
				;	//	wait until button is released
			finish = now + targetmin * 60000L;

			//	save minutes set to EEPROM
			EEPROM.write(EEPROMPOS, targetmin);
		}
	}
	else
	{	/*    countdown is running    */

		//	blink LED
		digitalWrite(pinLed, now%2000<1000);

		//	remaining minutes
		int remainingsec = (finish - now)/1000L;

		//	if DOWN is pressed, show seconds
		//	if UP is pressed, show minutes and seconds alternating
		if (isPressed(pinButtonDown) || (isPressed(pinButtonUp) && (remainingsec%2==0)))
			display(remainingsec%60);
		else
			display(remainingsec/60);

		//	if Start/Stop button is pressed, stop counting
		//	target min will be remaining minutes
		if (isPressed(pinButtonStart))
		{
			targetmin = remainingsec / 60;
			finish = 0;
			digitalWrite(pinLed, LOW);	//	be sure LED doesn't remain switched ON
			while (isPressed(pinButtonStart))
				;	//	wait until button is released
		}
	}
	delay(1);
}
